/* 
MathModule
For Resound 2.4
Sean Luke
Jan 20 1996

An example math transform module, including reversing, inverting,
abs, volume and vertical shifting.  This module includes a custom
nib file and several menu additions to Resound's Modules menu.

INTEL READY


Copyright (c) 1996 by Sean Luke

Permission to use, copy, modify, and distribute this material 
for any purpose and without fee is hereby granted.  SEAN LUKE 
MAKES NO REPRESENTATIONS ABOUT THE ACCURACY OR SUITABILITY 
OF THIS MATERIAL FOR ANY PURPOSE.  IT IS PROVIDED "AS IS", 
WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES.

*/

#import "MathModule.h"
#import <appkit/appkit.h>
#import <soundkit/soundkit.h>
#import <limits.h>
#import <math.h>

@implementation MathModule

-init
	{
	id returnVal=[super init];
	
	ModuleMenuNode* newNode=[[ModuleMenuNode alloc] init];
 	ModuleMenuNode* volumeNode=[[ModuleMenuNode alloc] init];
	ModuleMenuNode* shiftNode=[[ModuleMenuNode alloc] init];
	ModuleMenuNode* absNode=[[ModuleMenuNode alloc] init];
	ModuleMenuNode* invertNode=[[ModuleMenuNode alloc] init];
	ModuleMenuNode* reverseNode=[[ModuleMenuNode alloc] init];
	
	[TheModuleMenuNode setReceiver:NULL];		// has an item
	[TheModuleMenuNode setLength:2];
	[TheModuleMenuNode setSubmenu: 0 : newNode];
	[TheModuleMenuNode setSubmenu: 1 : reverseNode];
	
	[newNode setName: "Amplitude"];
	[newNode setReceiver: NULL];
	[newNode setLength:4];
	[newNode setSubmenu: 0 : volumeNode];
	[newNode setSubmenu: 1 : shiftNode];
	[newNode setSubmenu: 2 : invertNode];
	[newNode setSubmenu: 3 : absNode];
	
	[volumeNode setName: "Change Amplitude..."];
	[volumeNode setReceiver: self];
	[volumeNode setMessage:@selector(showMultiplyPanel:)];
	[volumeNode setLength:0];
		
	[shiftNode setName: "Vertical Shift..."];
	[shiftNode setReceiver: self];
	[shiftNode setMessage:@selector(showShiftPanel:)];
	[shiftNode setLength:0];
		
	[absNode setName: "Absolute Value"];
	[absNode setReceiver: self];
	[absNode setMessage:@selector(absIt:)];
	[absNode setLength:0];
		
	[invertNode setName: "Invert"];
	[invertNode setReceiver: self];
	[invertNode setMessage:@selector(invertIt:)];
	[invertNode setLength:0];
		
	[reverseNode setName: "Reverse"];
	[reverseNode setReceiver: self];
	[reverseNode setMessage:@selector(reverseIt:)];
	[reverseNode setLength:0];
			
	nib_loaded=0;
	return returnVal;
	}
	
	
	
- showMultiplyPanel:sender
	{
	char temp[MAXPATHLEN+1];
	
	if (!nib_loaded)
		{
		[[NXBundle bundleForClass: [MathModule class]]
			getPath:temp forResource:"Math" ofType:"nib"];
		[NXApp loadNibFile: temp owner:self withNames:NO fromZone:[self zone]];
		nib_loaded=1;
		}
	return [multiplyPanel makeKeyAndOrderFront:self];
	}

- showShiftPanel:sender
	{
	char temp[MAXPATHLEN+1];
	
	if (!nib_loaded)
		{
		[[NXBundle bundleForClass: [MathModule class]]
			getPath:temp forResource:"Math" ofType:"nib"];
		[NXApp loadNibFile: temp owner:self withNames:NO fromZone:[self zone]];
		nib_loaded=1;
		}
	return [shiftPanel makeKeyAndOrderFront:self];
	}

	
	
- reverseIt:sender
	{
	Sound* current=[TheModuleController currentSound];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];

	void* theData;
	unsigned char* data8;
	signed short* data16;
	
	int firstSample, sampleLength;
	
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];

	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	
	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{
		char trader;
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength/2)*channelCount;x++)
			{
			trader=data8[x];
			data8[x]=data8[(firstSample+sampleLength)*channelCount-x-1
				+firstSample*channelCount];
			data8[(firstSample+sampleLength)*channelCount-x-1
				+firstSample*channelCount]=trader;
			}
		if (channelCount==2)								// stereo
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x+=2)
			{
			trader=data8[x];
			data8[x]=data8[x+1];
			data8[x+1]=trader;
			}
		}
	else
		{
		short int trader;
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength/2)*channelCount;x++)
			{
			trader=data16[x];
			data16[x]=data16[(firstSample+sampleLength)*channelCount-x-1
				+firstSample*channelCount];
			data16[(firstSample+sampleLength)*channelCount-x-1
				+firstSample*channelCount]=trader;
			}
		if (channelCount==2)								// stereo
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x+=2)
			{
			trader=data16[x];
			data16[x]=data16[x+1];
			data16[x+1]=trader;
			}
		}
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	[TheModuleController soundChanged];
	return self;
	}
	
	
- absIt:sender
	{
	Sound* current=[TheModuleController currentSound];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];
	
	void* theData;
	unsigned char* data8;
	signed short* data16;
	
	int firstSample, sampleLength;
	
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];
		
	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);

	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{	
		int x;
		signed short tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=SNDiMulaw(data8[x]);
			tempsnd=abs(tempsnd);
			data8[x]=SNDMulaw(tempsnd);
			}
		}
	else
		{	
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			data16[x]=abs(data16[x]);
			}
		}
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	[TheModuleController soundChanged];
	return self;
	}
	

- invertIt:sender
	{
	Sound* current=[TheModuleController currentSound];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];
	
	void* theData;
	unsigned char* data8;
	signed short* data16;

	int firstSample, sampleLength;
		
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];

	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);

	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{	
		int x;
		signed short tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=SNDiMulaw(data8[x]);
			tempsnd=-tempsnd;
			data8[x]=SNDMulaw(tempsnd);
			}
		}
	else
		{	
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			data16[x]=-data16[x];
			}
		}
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	[TheModuleController soundChanged];
	return self;
	}
	

- multiplyIt:sender
	{
	Sound* current=[TheModuleController currentSound];
	int slider=[percentChangeField intValue];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];
	
	void* theData;
	unsigned char* data8;
	signed short* data16;
	
	int firstSample, sampleLength;
	
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];

	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);

	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{
		int x;
		int tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=SNDiMulaw(data8[x])*slider/100;
			if (tempsnd >  32767) tempsnd =  32767;
			if (tempsnd < -32768) tempsnd = -32768;
			data8[x]=SNDMulaw(tempsnd);
			}
		}
	else
		{
		int x;
		int tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=data16[x];
			tempsnd=tempsnd*slider/100;
			if (tempsnd >  32767) tempsnd =  32767;
			if (tempsnd < -32768) tempsnd = -32768;
			data16[x]=tempsnd;
			}
		}
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	[TheModuleController soundChanged];
	return self;
	}
			
- halve:sender
	{
	[percentChangeScroller setFloatValue:50];
	[percentChangeField setIntValue:50];	
	return self;
	}

- double:sender;
	{
	[percentChangeScroller setFloatValue:200];
	[percentChangeField setIntValue:200];	
	return self;
	}

- maximize:sender
	{
	Sound* current=[TheModuleController currentSound];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];
	
	void* theData;
	unsigned char* data8;
	signed short* data16;
	int max=0;
	
	int firstSample, sampleLength;
	
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];
		
	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);

	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{
		int x;
		int tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=SNDiMulaw(data8[x]);
			if (max<abs(tempsnd)) max=abs(tempsnd);
			}
		}
	else
		{
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			if (max<abs(data16[x])) max=abs(data16[x]);
			}
		}

	[percentChangeScroller setFloatValue:(int) (32767/(float)max*100)];
	[percentChangeField setFloatValue:(int) (32767/(float)max*100)];
	
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	return self;
	}	
	
- shiftIt:sender
	{
	Sound* current=[TheModuleController currentSound];
	float slider=[shiftSliderValue floatValue];
	
	int dataFormat=[current dataFormat];
	int channelCount=[current channelCount];
	int sampleCount=[current sampleCount];
	double samplingRate=[current samplingRate];
	
	void* theData;
	unsigned char* data8;
	signed short* data16;
	
	int firstSample, sampleLength;
	
	if (current==NULL)
		{
		NXRunAlertPanel("No Sound", 
			"There is no sound with which to perform this operation." , 
			"Okay",NULL,NULL);
		return self;
		}

	[TheModuleController stop];
	[current compactSamples];
		
	theData=(void*)[current data];
	data8=(char*)theData;
	data16=(signed short*)theData;

	SNDSwapSoundToHost
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);

	[[TheModuleController currentSoundView] 
		getSelection: &firstSample size: &sampleLength];
	if (!sampleLength) {firstSample=0;sampleLength=sampleCount;}	
		
	if ([current sampleSizePerChannel]==1)			// 8 bit sound
		{
		int x;
		int tempsnd;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			tempsnd=SNDiMulaw(data8[x]);
			tempsnd+=(slider*32767);
			if (tempsnd>SHRT_MAX) tempsnd=SHRT_MAX;
			if (tempsnd<SHRT_MIN) tempsnd=SHRT_MIN;
			data8[x]=SNDMulaw(tempsnd);
			}
		}
	else
		{
		int x;
		for (x=firstSample*channelCount;x<(firstSample+sampleLength)*channelCount;x++)
			{
			data16[x]+=(slider*32767);
			if (data16[x]>SHRT_MAX) data16[x]=SHRT_MAX;
			if (data16[x]<SHRT_MIN) data16[x]=SHRT_MIN;
			}
		}
		
	SNDSwapHostToSound
		((void*)theData, (void*)theData, sampleCount, channelCount, dataFormat);
	[TheModuleController soundChanged];
	return self;
	}
@end



